#include "c_wrapper.hpp"
#include "SignatureImage.hpp"
#include <cmath>
#include <cstring>
#include <iostream>

namespace signiamge::c_wrapper {
Result *getResult(Params params) noexcept {
    if (params.title == nullptr || params.cert_serial == nullptr ||
        params.subject == nullptr || params.time_validity == nullptr) {

        return nullptr;
    }

    try {
        signimage::CertificateInfo certInfo = {
            params.cert_serial, params.subject, params.time_validity};

        std::unique_ptr<signimage::SignatureImage> sig;

        bool withLogo = true;
        if (params.ptr_logo_data != nullptr) {
            sig = std::make_unique<signimage::SignatureImage>(
                params.title, params.ptr_logo_data, params.ptr_logo_size,
                &certInfo);
        } else if (params.ptr_logo_file != nullptr) {
            sig = std::make_unique<signimage::SignatureImage>(
                params.title, params.ptr_logo_file, &certInfo);
        } else {
            sig = std::make_unique<signimage::SignatureImage>(params.title,
                                                              &certInfo);
            withLogo = false;
        }

        sig->setSignatureSize(
            {params.signature_size.width, params.signature_size.height});

        sig->setBackgroundColor({params.bg_color.r, params.bg_color.g,
                                 params.bg_color.b, params.bg_color.alpha});

        sig->setTextColor({params.text_color.r, params.text_color.g,
                           params.text_color.b, params.text_color.alpha});

        sig->setBorderColor({params.border_color.r, params.border_color.g,
                             params.border_color.b, params.border_color.alpha});

        sig->setBorderRadius({params.border_radius.cornerWidth,
                              params.border_radius.cornerHeight});

        if (withLogo) {
            std::string logoScale =
                std::to_string((size_t)floor(params.logo_size_goal.x)) + "x" +
                std::to_string((size_t)floor(params.logo_size_goal.y));
            sig->setLogoScale(logoScale);

            sig->setLogoPreserveRation(params.logo_preserve_ratio);

            sig->setLogoPos({params.logo_position.x, params.logo_position.y});
        }

        sig->setTitlePos({params.title_position.x, params.title_position.y});

        sig->setCertSerialPos(
            {params.cert_serial_position.x, params.cert_serial_position.y});

        sig->setSubjectPos(
            {params.subject_position.x, params.subject_position.y});

        sig->setTimeValidityPos(
            {params.time_validity_position.x, params.time_validity_position.y});

        sig->setBorderWidth(params.border_width);

        sig->setFontFamily(params.font_family);

        sig->setTitleFontSize(params.title_font_size);

        sig->setFontSize(params.font_size);

        sig->setRightMargin(params.right_margin);

        auto titleAlignment =
            static_cast<signimage::SignatureImage::TextAlignment>(
                params.title_alignment);
        sig->setTitleAlignment(titleAlignment);

        auto contentAlignment =
            static_cast<signimage::SignatureImage::TextAlignment>(
                params.content_alignment);

        sig->draw(static_cast<signimage::SignatureImage::ColorModel>(
            params.color_model_out));

        if (sig->getRawData().empty()) {
#ifndef NDEBUG
            std::cerr << "The signature image could not be created!"
                      << std::endl;
#endif

            return nullptr;
        }

        Result *result = new Result;

        auto [sigResolutionX, sigResolutionY] = sig->getSignatureResolution();
        result->resolution = {(size_t)floor(sigResolutionX),
                              (size_t)floor(sigResolutionY)};

        auto rawData = sig->getRawData();

        result->stamp_img_data_size = rawData.size();

        result->stamp_img_data = new unsigned char[result->stamp_img_data_size];
        memcpy(result->stamp_img_data, rawData.data(),
               result->stamp_img_data_size);

        return result;
    } catch (std::exception &err) {
#ifdef NDEBUG
        std::cerr << "Caught exception: " << err.what() << std::endl;
#endif

        return nullptr;
    }
}

void FreeResult(Result *ptr) noexcept {
    if (ptr != nullptr) {
        delete[] ptr->stamp_img_data;
    }
    delete ptr;
}
} // namespace signiamge::c_wrapper
